/*  VT1500 Device Driver for Linux, Version 1.0.10
 *  Copyright (C) 1995-2001  M. Gutschke
 *
 *  At the time of writing my e-mail address is:
 *	Internet: markus@gutschke.com
 *  My snail mail address is:
 *	Markus Gutschke
 *	3637 Fillmore Street #106
 *	San Francisco, CA 94123-1600
 *	USA
 *  If you like this software, I would appreciate if you sent me a postcard
 *  from your hometown. Under the terms of the GNU general public license
 *  you are free to include this program into (commercial) software
 *  distributions (e.g. putting it onto CD-ROM); nonetheless, I would really
 *  appreciate if you dropped me a short note (sending me a sample copy of
 *  your distribution would be even more appreciated!)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "cmds.h"
#include "server.h"

extern const char gpl[];
extern const char authormsg[];

void delParms(ParmRec *parmList)
{
  while (parmList) {
    ParmRec *tmp = parmList->next;
    if (parmList->type == tStr &&
	parmList->parm.sParm)
      free(parmList->parm.sParm);
    free(parmList);
    parmList = tmp; }
  return;
}

static int getParms(ParmRec *parmList,const char *fmt,...)
{
  int     rc = 0;
  va_list arg_ptr;

  va_start(arg_ptr,fmt);
  for (; *fmt && parmList; fmt++, parmList = parmList->next)
    switch (*fmt) {
    case 'i':
      if (parmList->type != tInt) {
	serverror("Integer expected"); goto err; }
      *va_arg(arg_ptr,int *) = parmList->parm.iParm;
      break;
    case 's':
      if (parmList->type != tStr) {
	serverror("String expected"); goto err; }
      strcpy(va_arg(arg_ptr,char *),parmList->parm.sParm);
      break;
    default:
      serverror("Internal error");
      goto err; }
  if (!*fmt != !parmList) {
    if (!*fmt) serverror("Too many parameters");
    else       serverror("Missing parameters"); }
  else
    rc = 1;
err:
  va_end(arg_ptr);
  return(rc);
}

void servAuthor(ParmRec *parmList)
{
  if (!getParms(parmList,"")) return;
  writeClient(authormsg,strlen(authormsg));
  return;
}

void servWarranty(ParmRec *parmList)
{
  char *ptr = strstr(gpl,"NO WARRANTY")-1;

  if (!getParms(parmList,"")) return;
  while ((*ptr == ' ') || (*ptr == ('I'&0x1F))) ptr--;
  writeClient(ptr+1,index(ptr,'')-ptr-1);
  return;
}

void servConditions(ParmRec *parmList)
{
  if (!getParms(parmList,"")) return;
  writeClient(gpl,strlen(gpl));
  return;
}

void servHelp(ParmRec *parmList)
{
  static const char msg[] =
    "Available commands:\n"
    "  author();            -- display information about author\n"
    "  channelInfo(int idx,int chPr,int inc);\n"
    "  channelName(int frq);-- return channel name and index (48..855MHz)\n"
    "  clearScreen();       -- clear on-screen display\n"
    "  conditions();        -- display license conditions\n"
    "  getBass();           -- return bass\n"
    "  getBlue();           -- return blue\n"
    "  getBrightness();     -- return brightness\n"
    "  getChannel();        -- return currently active channel\n"
    "  getContrast();       -- return contrast\n"
    "  getExternalMute();   -- check how muting is implemented\n"
    "  getExternalVolume(); -- return external volume\n"
    "  getFrq();            -- return currently active frequency\n"
    "  getGreen();          -- return green\n"
    "  getHue();            -- return hue\n"
    "  getInternalVolume(); -- return internal volume\n"
    "  getMute();           -- check if muting is active\n"
    "  tmpMute();           -- temporarily disable sound\n"
    "  refreshMute();       -- restore sound after temporary muting\n"
    "  getPALNTSC();        -- check TV encoding mode\n"
    "  getRed();            -- return red\n"
    "  getSaturation();     -- return saturation\n"
    "  getTVVCR();          -- get source information\n"
    "  getTreble();         -- return treble\n"
    "  getVolume();         -- return volume\n"
    "  help();              -- display this message\n"
    "  isRadio();           -- check if radio mode has been enabled\n"
    "  isVisible();         -- check if tv or vga mode is active\n"
    "  kill();              -- kill server\n"
    "  quit();              -- terminate connection\n"
    "  setBass(int bass)    -- change bass (0..15)\n"
    "  setBlue(int blue)    -- blue (0..63)\n"
    "  setBrightness(int b) -- set brightness (0..63)\n"
    "  setChannel(int ch)   -- switch to channel (0..99)\n"
    "  setChannelInfo(int idx,char *name,int adj,int prg,int PALNTSC)\n"
    "  setContrast(int ctr) -- set contrast (0..63)\n"
    "  setFrq(int frq)      -- set frequency in units of 100kHz (48..855MHz)\n"
    "  setGreen(int green)  -- green (0..63)\n"
    "  setHue(int hue)      -- set Hue (0..255)\n"
    "  setMute(int flag)    -- en-/disable muting\n"
    "  setNTSC()            -- enable NTSC mode\n"
    "  setPAL()             -- enable PAL mode\n"
    "  setRadio(int flag)   -- select radio/tv mode\n"
    "  setRed(int red)      -- red (0..63)\n"
    "  setSaturation(int s) -- set saturation (0..63)\n"
    "  setTV()              -- select tuner for input source\n"
    "  setTreble(int trb)   -- change treble (0..15)\n"
    "  setVCR()             -- select FBAS signal for input source\n"
    "  setVGA(int flag)     -- switch between VGA/TV mode \n"
    "  setVolume(int vol)   -- change volume (0..31)\n"
    "  showBass(int bass);  -- show bass on display\n"
    "  showChannel(int,char *,char *);\n"
    "  showColor(int,...);  -- show color settings on display\n"
    "  showHelp();          -- show help screen\n"
    "  showMute(int flag);  -- show muting symbol\n"
    "  showTVVCR(int flag); -- show source selection display\n"
    "  showTreble(int trb); -- show treble on display\n"
    "  showVolume(int vol); -- show volume on display\n"
    "  sndCmd(int cmd)      -- write command to I2C bus\n"
    "  warp(int vt);        -- warp to virtual console\n"
    "  warranty();          -- display warranty information\n"
    ;

  if (!getParms(parmList,"")) return;
  writeClient(msg,strlen(msg));
  return;
}

void servQuit(ParmRec *parmList)
{
  if (!getParms(parmList,"")) return;
  exit(0);
}

void servKill(ParmRec *parmList)
{
  ReqRec req = {0,Okill,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servWarp(ParmRec *parmList)
{
  ReqRec req = {0,Owarp,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSndCmd(ParmRec *parmList)
{
  ReqRec req = {0,OsndCmd,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetRadio(ParmRec *parmList)
{
  ReqRec req = {0,OsetRadio,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetFrq(ParmRec *parmList)
{
  ReqRec req = {0,OsetFrq,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetChannel(ParmRec *parmList)
{
  ReqRec req = {0,OsetChannel,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetVolume(ParmRec *parmList)
{
  ReqRec req = {0,OsetVolume,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetBass(ParmRec *parmList)
{
  ReqRec req = {0,OsetBass,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetTreble(ParmRec *parmList)
{
  ReqRec req = {0,OsetTreble,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetMute(ParmRec *parmList)
{
  ReqRec req = {0,OsetMute,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servTmpMute(ParmRec *parmList)
{
  ReqRec req = {0,OtmpMute,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servRefreshMute(ParmRec *parmList)
{
  ReqRec req = {0,OrefreshMute,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servSetVGA(ParmRec *parmList)
{
  ReqRec req = {0,OsetVGA,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetTV(ParmRec *parmList)
{
  ReqRec req = {0,OsetTV,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servSetVCR(ParmRec *parmList)
{
  ReqRec req = {0,OsetVCR};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servSetPAL(ParmRec *parmList)
{
  ReqRec req = {0,OsetPAL,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servSetNTSC(ParmRec *parmList)
{
  ReqRec req = {0,OsetNTSC,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servSetHue(ParmRec *parmList)
{
  ReqRec req = {0,OsetHue,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetBrightness(ParmRec *parmList)
{
  ReqRec req = {0,OsetBrightness,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetSaturation(ParmRec *parmList)
{
  ReqRec req = {0,OsetSaturation,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetContrast(ParmRec *parmList)
{
  ReqRec req = {0,OsetContrast,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetRed(ParmRec *parmList)
{
  ReqRec req = {0,OsetRed,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetGreen(ParmRec *parmList)
{
  ReqRec req = {0,OsetGreen,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servSetBlue(ParmRec *parmList)
{
  ReqRec req = {0,OsetBlue,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servChannelName(ParmRec *parmList)
{
  ReqRec req = {0,OchannelName,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servIsVisible(ParmRec *parmList)
{
  ReqRec req = {0,OisVisible,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servIsRadio(ParmRec *parmList)
{
  ReqRec req = {0,OisRadio,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetTVVCR(ParmRec *parmList)
{
  ReqRec req = {0,OgetTVVCR,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetPALNTSC(ParmRec *parmList)
{
  ReqRec req = {0,OgetPALNTSC,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetChannel(ParmRec *parmList)
{
  ReqRec req = {0,OgetChannel,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetFrq(ParmRec *parmList)
{
  ReqRec req = {0,OgetFrq,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetMute(ParmRec *parmList)
{
  ReqRec req = {0,OgetMute,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetExternalMute(ParmRec *parmList)
{
  ReqRec req = {0,OgetExternalMute,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetBass(ParmRec *parmList)
{
  ReqRec req = {0,OgetBass,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetTreble(ParmRec *parmList)
{
  ReqRec req = {0,OgetTreble,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetVolume(ParmRec *parmList)
{
  ReqRec req = {0,OgetVolume,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetInternalVolume(ParmRec *parmList)
{
  ReqRec req = {0,OgetInternalVolume,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetExternalVolume(ParmRec *parmList)
{
  ReqRec req = {0,OgetExternalVolume,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetBrightness(ParmRec *parmList)
{
  ReqRec req = {0,OgetBrightness,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetSaturation(ParmRec *parmList)
{
  ReqRec req = {0,OgetSaturation,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetContrast(ParmRec *parmList)
{
  ReqRec req = {0,OgetContrast,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetHue(ParmRec *parmList)
{
  ReqRec req = {0,OgetHue,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetRed(ParmRec *parmList)
{
  ReqRec req = {0,OgetRed,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetGreen(ParmRec *parmList)
{
  ReqRec req = {0,OgetGreen,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servGetBlue(ParmRec *parmList)
{
  ReqRec req = {0,OgetBlue,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servChannelInfo(ParmRec *parmList)
{
  ReqRec req = {0,OchannelInfo,};

  if (!getParms(parmList,"iii",&req.parm.iParms[0],&req.parm.iParms[1],
		&req.parm.iParms[2])) return;
  sendReq(&req); return;
}

void servSetChannelInfo(ParmRec *parmList)
{
  ReqRec req = {0,OsetChannelInfo,};

  if (!getParms(parmList,"isiii",&req.parm.x.iParms[0],req.parm.x.sParm,
		&req.parm.x.iParms[1],&req.parm.x.iParms[2],
		&req.parm.x.iParms[3]))
    return;
  sendReq(&req); return;
}

void servClearScreen(ParmRec *parmList)
{
  ReqRec req = {0,OclearScreen,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servShowHelp(ParmRec *parmList)
{
  ReqRec req = {0,OshowHelp,};

  if (!getParms(parmList,"")) return;
  sendReq(&req); return;
}

void servShowTVVCR(ParmRec *parmList)
{
  ReqRec req = {0,OshowTVVCR,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servShowMute(ParmRec *parmList)
{
  ReqRec req = {0,OshowMute,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servShowBass(ParmRec *parmList)
{
  ReqRec req = {0,OshowBass,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servShowTreble(ParmRec *parmList)
{
  ReqRec req = {0,OshowTreble,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servShowVolume(ParmRec *parmList)
{
  ReqRec req = {0,OshowVolume,};

  if (!getParms(parmList,"i",&req.parm.iParm)) return;
  sendReq(&req); return;
}

void servShowColor(ParmRec *parmList)
{
  ReqRec req = {0,OshowColor,};

  if (!getParms(parmList,"iiiiiiii",&req.parm.iParms[0],&req.parm.iParms[1],
		&req.parm.iParms[2],&req.parm.iParms[3],&req.parm.iParms[4],
		&req.parm.iParms[5],&req.parm.iParms[6],&req.parm.iParms[7]))
    return;
  sendReq(&req); return;
}

void servShowChannel(ParmRec *parmList)
{
  ReqRec req = {0,OshowChannel,};

  if (!getParms(parmList,"iss",&req.parm.y.iParms[0],&req.parm.y.sParm1,
		&req.parm.y.sParm2)) return;
  sendReq(&req); return;
}

